# - Find GraphViz
# Find the native GraphViz headers and libraries.
#
# GRAPHVIZ_INCLUDE_DIRS
# GRAPHVIZ_LIBRARIES
# GRAPHVIZ_FOUND


FIND_PATH(GRAPHVIZ_INCLUDE_DIR NAMES graphviz/cgraph.h graphviz/graphviz_version.h)
FIND_LIBRARY(GRAPHVIZ_GVC_LIBRARY NAMES gvc)
FIND_LIBRARY(GRAPHVIZ_CDT_LIBRARY NAMES cdt)
FIND_LIBRARY(GRAPHVIZ_PATHPLAN_LIBRARY NAMES pathplan)
#FIND_LIBRARY(GRAPHVIZ_GRAPH_LIBRARY NAMES graph)

if(GRAPHVIZ_INCLUDE_DIR)
	file(READ "${GRAPHVIZ_INCLUDE_DIR}/graphviz/graphviz_version.h" GRAPHVIZ_VERSION_FILE)
	string(REGEX MATCH "PACKAGE_VERSION \"(.*)\"" _ ${GRAPHVIZ_VERSION_FILE})
	SET(GRAPHVIZ_VERSION ${CMAKE_MATCH_1})
else()
	SET(GRAPHVIZ_VERSION)
endif()

# Handle the QUIETLY and REQUIRED arguments and set GRAPHVIZ_FOUND to TRUE if all listed variables are TRUE.
INCLUDE(FindPackageHandleStandardArgs)
FIND_PACKAGE_HANDLE_STANDARD_ARGS(Graphviz
	FOUND_VAR GRAPHVIZ_FOUND
	REQUIRED_VARS GRAPHVIZ_GVC_LIBRARY GRAPHVIZ_CDT_LIBRARY GRAPHVIZ_PATHPLAN_LIBRARY GRAPHVIZ_INCLUDE_DIR
	VERSION_VAR GRAPHVIZ_VERSION
	)

# Copy the results to the output variables.
IF(GRAPHVIZ_FOUND)
	SET(GRAPHVIZ_LIBRARIES ${GRAPHVIZ_GVC_LIBRARY} ${GRAPHVIZ_CDT_LIBRARY} ${GRAPHVIZ_PATHPLAN_LIBRARY})
	SET(GRAPHVIZ_INCLUDE_DIRS ${GRAPHVIZ_INCLUDE_DIR})
ELSE(GRAPHVIZ_FOUND)
	SET(GRAPHVIZ_LIBRARIES)
	SET(GRAPHVIZ_INCLUDE_DIRS)
ENDIF(GRAPHVIZ_FOUND)

MARK_AS_ADVANCED(GRAPHVIZ_INCLUDE_DIRS GRAPHVIZ_LIBRARIES)
